"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-non-null-assertion */
const assert_1 = require("assert");
const crypto = __importStar(require("crypto"));
const internal_1 = require("@fluidframework/container-runtime-previous/internal");
const internal_2 = require("@fluidframework/telemetry-utils/internal");
const containerRuntime_js_1 = require("../../containerRuntime.js");
const index_js_1 = require("../../opLifecycle/index.js");
function typeFromBatchedOp(op) {
    (0, assert_1.strict)(op.contents !== undefined);
    return JSON.parse(op.contents).type;
}
describe("OpSplitter", () => {
    const batchesSubmitted = [];
    const mockSubmitBatchFn = (batch, referenceSequenceNumber) => {
        batchesSubmitted.push({ messages: batch, referenceSequenceNumber });
        return batchesSubmitted.length;
    };
    beforeEach(() => {
        batchesSubmitted.splice(0);
        mockLogger.clear();
    });
    const chunkSizeInBytes = 50 * 1024;
    const maxBatchSizeInBytes = 10 * 50 * 1024;
    const mockLogger = new internal_2.MockLogger();
    it("Reconstruct original chunked op", () => {
        const op1 = generateChunkableOp(chunkSizeInBytes * 2);
        const op2 = generateChunkableOp(chunkSizeInBytes * 3);
        const chunks1 = wrapChunkedOps((0, index_js_1.splitOp)(op1, chunkSizeInBytes), "testClient1");
        const chunks2 = wrapChunkedOps((0, index_js_1.splitOp)(op2, chunkSizeInBytes), "testClient2");
        const opSplitter = new index_js_1.OpSplitter([], mockSubmitBatchFn, 0, maxBatchSizeInBytes, mockLogger);
        assert_1.strict.equal(opSplitter.processChunk(chunks1[0]).isFinalChunk, false);
        assert_1.strict.equal(opSplitter.processChunk(chunks2[0]).isFinalChunk, false);
        assert_1.strict.equal(opSplitter.processChunk(chunks1[1]).isFinalChunk, false);
        assert_1.strict.equal(opSplitter.processChunk(chunks2[1]).isFinalChunk, false);
        const chunks1LastResult = opSplitter.processChunk(chunks1[2]);
        // The last chunk will reconstruct the original message
        assert_1.strict.equal(chunks1LastResult.isFinalChunk, true);
        assertSameMessage(chunks1LastResult.message, op1);
        assert_1.strict.equal(opSplitter.chunks.size, 1);
        assert_1.strict.equal(opSplitter.processChunk(chunks2[2]).isFinalChunk, false);
        const chunks2LastResult = opSplitter.processChunk(chunks2[3]);
        // The last chunk will reconstruct the original message
        assert_1.strict.equal(chunks2LastResult.isFinalChunk, true);
        assertSameMessage(chunks2LastResult.message, op2);
        assert_1.strict.equal(opSplitter.chunks.size, 0);
    });
    it("Reconstruct original chunked op with extra empty op", () => {
        const op1 = generateChunkableOp(chunkSizeInBytes * 2);
        const op2 = generateChunkableOp(chunkSizeInBytes * 3);
        const chunks1 = wrapChunkedOps((0, index_js_1.splitOp)(op1, chunkSizeInBytes, true), "testClient1");
        const chunks2 = wrapChunkedOps((0, index_js_1.splitOp)(op2, chunkSizeInBytes, true), "testClient2");
        const opSplitter = new index_js_1.OpSplitter([], mockSubmitBatchFn, 0, maxBatchSizeInBytes, mockLogger);
        assert_1.strict.equal(opSplitter.processChunk(chunks1[0]).isFinalChunk, false);
        assert_1.strict.equal(opSplitter.processChunk(chunks2[0]).isFinalChunk, false);
        assert_1.strict.equal(opSplitter.processChunk(chunks1[1]).isFinalChunk, false);
        assert_1.strict.equal(opSplitter.processChunk(chunks2[1]).isFinalChunk, false);
        assert_1.strict.equal(opSplitter.processChunk(chunks1[2]).isFinalChunk, false);
        const chunks1LastResult = opSplitter.processChunk(chunks1[3]);
        // The last chunk will reconstruct the original message
        assert_1.strict.equal(chunks1LastResult.isFinalChunk, true);
        assertSameMessage(chunks1LastResult.message, op1);
        assert_1.strict.equal(opSplitter.chunks.size, 1);
        assert_1.strict.equal(opSplitter.processChunk(chunks2[2]).isFinalChunk, false);
        assert_1.strict.equal(opSplitter.processChunk(chunks2[3]).isFinalChunk, false);
        const chunks2LastResult = opSplitter.processChunk(chunks2[4]);
        // The last chunk will reconstruct the original message
        assert_1.strict.equal(chunks2LastResult.isFinalChunk, true);
        assertSameMessage(chunks2LastResult.message, op2);
        assert_1.strict.equal(opSplitter.chunks.size, 0);
    });
    it("Reconstruct original chunked op with initial chunks", () => {
        const op = generateChunkableOp(chunkSizeInBytes * 3);
        const chunks = wrapChunkedOps((0, index_js_1.splitOp)(op, chunkSizeInBytes), "testClient");
        const opSplitter = new index_js_1.OpSplitter([], mockSubmitBatchFn, 0, maxBatchSizeInBytes, mockLogger);
        opSplitter.processChunk(chunks[0]);
        opSplitter.processChunk(chunks[1]);
        const otherOpSplitter = new index_js_1.OpSplitter(Array.from(opSplitter.chunks), mockSubmitBatchFn, 0, maxBatchSizeInBytes, mockLogger);
        opSplitter.clearPartialChunks("testClient");
        otherOpSplitter.processChunk(chunks[2]);
        const processResult = otherOpSplitter.processChunk(chunks[3]);
        assert_1.strict.equal(processResult.isFinalChunk, true);
        assertSameMessage(processResult.message, op);
    });
    it("Clear chunks", () => {
        const chunks = wrapChunkedOps((0, index_js_1.splitOp)(generateChunkableOp(chunkSizeInBytes * 3), chunkSizeInBytes), "testClient");
        const opSplitter = new index_js_1.OpSplitter([], mockSubmitBatchFn, 0, maxBatchSizeInBytes, mockLogger);
        opSplitter.processChunk(chunks[0]);
        assert_1.strict.equal(opSplitter.chunks.size, 1);
        opSplitter.clearPartialChunks("noClient");
        assert_1.strict.equal(opSplitter.chunks.size, 1);
        opSplitter.clearPartialChunks("testClient");
        assert_1.strict.equal(opSplitter.chunks.size, 0);
    });
    it("Throw when processing out-of-order chunks", () => {
        const chunks = wrapChunkedOps((0, index_js_1.splitOp)(generateChunkableOp(chunkSizeInBytes * 3), chunkSizeInBytes), "testClient1");
        const opSplitter = new index_js_1.OpSplitter([], mockSubmitBatchFn, 0, maxBatchSizeInBytes, mockLogger);
        assert_1.strict.throws(() => opSplitter.processChunk(chunks[2]));
    });
    it("Don't accept non-chunked ops", () => {
        const chunks = wrapChunkedOps((0, index_js_1.splitOp)(generateChunkableOp(chunkSizeInBytes * 3), chunkSizeInBytes), "testClient1").map((op) => {
            op.contents.type = internal_1.ContainerMessageType.FluidDataStoreOp;
            return op;
        });
        const opSplitter = new index_js_1.OpSplitter([], mockSubmitBatchFn, 0, maxBatchSizeInBytes, mockLogger);
        for (const op of chunks) {
            assert_1.strict.deepStrictEqual((0, index_js_1.isChunkedMessage)(op), false);
            assert_1.strict.throws(() => opSplitter.processChunk(op));
        }
    });
    it("Chunk metadata", () => {
        const originalOp = generateChunkableOp(chunkSizeInBytes * 4);
        const chunks = (0, index_js_1.splitOp)(originalOp, chunkSizeInBytes);
        assert_1.strict.equal(chunks
            .slice(0, -1)
            .every((chunk) => chunk.originalCompression === undefined && chunk.originalMetadata === undefined), true);
        const lastChunk = chunks[chunks.length - 1];
        assert_1.strict.deepStrictEqual(lastChunk.originalMetadata, originalOp.metadata);
        assert_1.strict.equal(lastChunk.originalCompression, originalOp.compression);
    });
    it("Batch split invariants", () => {
        const opSplitter = new index_js_1.OpSplitter([], mockSubmitBatchFn, 50, maxBatchSizeInBytes, mockLogger);
        const regularMessage = generateChunkableOp(20);
        const compressedMessage = { ...regularMessage, metadata: { compressed: true } };
        // Empty batch
        assert_1.strict.throws(() => opSplitter.splitFirstBatchMessage({
            messages: [compressedMessage],
            contentSizeInBytes: 0,
            referenceSequenceNumber: 0,
        }));
        // Empty batch
        assert_1.strict.throws(() => opSplitter.splitFirstBatchMessage({
            messages: [],
            contentSizeInBytes: 1,
            referenceSequenceNumber: 0,
        }));
        // Batch is too small to be chunked
        assert_1.strict.throws(() => opSplitter.splitFirstBatchMessage({
            messages: [compressedMessage],
            contentSizeInBytes: 1,
            referenceSequenceNumber: 0,
        }));
        // Batch is not compressed
        assert_1.strict.throws(() => opSplitter.splitFirstBatchMessage({
            messages: [regularMessage],
            contentSizeInBytes: 3,
            referenceSequenceNumber: 0,
        }));
        // Misconfigured op splitter
        assert_1.strict.throws(() => new index_js_1.OpSplitter([], mockSubmitBatchFn, 0, maxBatchSizeInBytes, mockLogger).splitFirstBatchMessage({
            messages: [compressedMessage],
            contentSizeInBytes: 3,
            referenceSequenceNumber: 0,
        }));
        // Old loader
        assert_1.strict.throws(() => new index_js_1.OpSplitter([], undefined, 0, maxBatchSizeInBytes, mockLogger).splitFirstBatchMessage({
            messages: [compressedMessage],
            contentSizeInBytes: 3,
            referenceSequenceNumber: 0,
        }));
        // Misconfigured op splitter
        assert_1.strict.throws(() => new index_js_1.OpSplitter([], mockSubmitBatchFn, 2, 1, mockLogger).splitFirstBatchMessage({
            messages: [compressedMessage],
            contentSizeInBytes: 3,
            referenceSequenceNumber: 0,
        }));
        // Not enabled
        assert_1.strict.throws(() => new index_js_1.OpSplitter([], mockSubmitBatchFn, Number.POSITIVE_INFINITY, maxBatchSizeInBytes, mockLogger).splitFirstBatchMessage({
            messages: [compressedMessage],
            contentSizeInBytes: 3,
            referenceSequenceNumber: 0,
        }));
    });
    describe("Compressed batches", () => {
        [false, true].forEach((extraOp) => {
            it(`Split compressed batch with multiple messages with${extraOp ? "" : "out"} extra empty op.`, () => {
                const chunkSize = 20;
                const opSplitter = new index_js_1.OpSplitter([], mockSubmitBatchFn, chunkSize, extraOp ? chunkSize * 2 : maxBatchSizeInBytes, mockLogger);
                const largeMessage = generateChunkableOp(100);
                const emptyMessage = generateChunkableOp(0);
                const result = opSplitter.splitFirstBatchMessage({
                    messages: [largeMessage, emptyMessage, emptyMessage, emptyMessage],
                    contentSizeInBytes: largeMessage.contents?.length ?? 0,
                    referenceSequenceNumber: 0,
                });
                assert_1.strict.equal(batchesSubmitted.length, 5 + (extraOp ? 1 : 0));
                for (const batch of batchesSubmitted) {
                    assert_1.strict.equal(batch.messages.length, 1);
                    assert_1.strict.equal(typeFromBatchedOp(batch.messages[0]), internal_1.ContainerMessageType.ChunkedOp);
                    assert_1.strict.equal(batch.referenceSequenceNumber, 0);
                }
                assert_1.strict.equal(result.messages.length, 4);
                const lastChunk = JSON.parse(result.messages[0].contents).contents;
                assert_1.strict.equal(lastChunk.chunkId, lastChunk.totalChunks);
                assert_1.strict.deepStrictEqual(result.messages.slice(1), new Array(3).fill(emptyMessage));
                assert_1.strict.equal(!extraOp ||
                    JSON.parse(result.messages[0].contents).contents?.contents?.length === 0, true);
                assert_1.strict.notEqual(result.contentSizeInBytes, largeMessage.contents?.length ?? 0);
                const contentSentSeparately = batchesSubmitted.map((x) => JSON.parse(x.messages[0].contents).contents
                    .contents);
                const sentContent = [...contentSentSeparately, lastChunk.contents].reduce((accumulator, current) => `${accumulator}${current}`);
                assert_1.strict.equal(sentContent, largeMessage.contents);
                (0, assert_1.strict)(mockLogger.matchEvents([
                    {
                        eventName: "OpSplitter:CompressedChunkedBatch",
                        length: result.messages.length,
                        chunks: 100 / 20 + 1 + (extraOp ? 1 : 0),
                        chunkSizeInBytes: 20,
                    },
                ]));
            });
            it(`Split compressed batch with single message with${extraOp ? "" : "out"} extra empty op.`, () => {
                const chunkSize = 20;
                const opSplitter = new index_js_1.OpSplitter([], mockSubmitBatchFn, 20, extraOp ? chunkSize * 2 : maxBatchSizeInBytes, mockLogger);
                const largeMessage = generateChunkableOp(100);
                const result = opSplitter.splitFirstBatchMessage({
                    messages: [largeMessage],
                    contentSizeInBytes: largeMessage.contents?.length ?? 0,
                    referenceSequenceNumber: 0,
                });
                assert_1.strict.equal(batchesSubmitted.length, 5 + (extraOp ? 1 : 0));
                for (const batch of batchesSubmitted) {
                    assert_1.strict.equal(batch.messages.length, 1);
                    assert_1.strict.equal(typeFromBatchedOp(batch.messages[0]), internal_1.ContainerMessageType.ChunkedOp);
                    assert_1.strict.equal(batch.referenceSequenceNumber, 0);
                }
                assert_1.strict.equal(result.messages.length, 1);
                assert_1.strict.notEqual(result.contentSizeInBytes, largeMessage.contents?.length ?? 0);
                const lastChunk = JSON.parse(result.messages[0].contents).contents;
                assert_1.strict.equal(lastChunk.chunkId, lastChunk.totalChunks);
                assert_1.strict.equal(!extraOp ||
                    JSON.parse(result.messages[0].contents).contents?.contents?.length === 0, true);
                assert_1.strict.notEqual(result.contentSizeInBytes, largeMessage.contents?.length ?? 0);
                const contentSentSeparately = batchesSubmitted.map((x) => JSON.parse(x.messages[0].contents).contents
                    .contents);
                const sentContent = [...contentSentSeparately, lastChunk.contents].reduce((accumulator, current) => `${accumulator}${current}`);
                assert_1.strict.equal(sentContent, largeMessage.contents);
                (0, assert_1.strict)(mockLogger.matchEvents([
                    {
                        eventName: "OpSplitter:CompressedChunkedBatch",
                        length: result.messages.length,
                        chunks: 100 / 20 + 1 + (extraOp ? 1 : 0),
                        chunkSizeInBytes: 20,
                    },
                ]));
            });
        });
    });
    const assertSameMessage = (result, original) => {
        assert_1.strict.deepStrictEqual(result.contents, JSON.parse(original.contents));
        // type = "component" is used to force 1.3 to crash on compressed & chunked ops, as it does not understand it.
        // 2.x does not care about type, as it will get right type after decompressing the op.
        // see code & comment in splitOp()
        assert_1.strict.strictEqual(result.type, "component");
        assert_1.strict.strictEqual(result.metadata, original.metadata);
        assert_1.strict.strictEqual(result.compression, original.compression);
    };
    const generateChunkableOp = (contentSizeInBytes) => {
        const contents = {
            // There should be a type here, but there is no validation for that,
            // and tests would need to be adjusted (sizing and assumptions) if we add it here.
            // type: ContainerMessageType.FluidDataStoreOp,
            value: crypto.randomBytes(contentSizeInBytes / 2).toString("hex"),
        };
        return {
            referenceSequenceNumber: Infinity,
            metadata: { meta: "data" },
            compression: containerRuntime_js_1.CompressionAlgorithms.lz4,
            contents: JSON.stringify(contents),
        };
    };
    const wrapChunkedOps = (ops, clientId) => ops.map((op) => {
        const result = {
            contents: {
                type: internal_1.ContainerMessageType.ChunkedOp,
                contents: op,
            },
            clientId,
        };
        return result;
    });
});
//# sourceMappingURL=opSplitter.spec.js.map